@@ -216,7 +216,10 @@ class Scheduler:
         }
 
         # Create a list of initial files to scan with the fast REGEX frontend
-        path_list = [path.glob(f'**/*{ext}') for path in self.paths for ext in self.source_suffixes]
+        path_list = [
+            path.glob(f'**/*{ext}') if path.is_dir() else path
+            for path in self.paths for ext in self.source_suffixes
+        ]
         path_list = list(set(flatten(path_list)))  # Filter duplicates and flatten
 
         # Instantiate FileItem instances for all files in the search path
@@ -552,7 +555,7 @@ class Scheduler:
 
                 transformation.apply(
                     _item.scope_ir, item=_item, items=_get_definition_items(_item, sgraph_items),
-                    successors=graph.successors(_item, item_filter=item_filter),
+                    sub_sgraph=graph.get_sub_sgraph(_item, item_filter=item_filter),
                     role=_item.role, mode=_item.mode, targets=_item.targets,
                     **kwargs
                 )
