@@ -20,10 +20,7 @@ except ImportError:
 
 from loki.logging import debug, warning
 
-__all__ = [
-    'Source', 'FortranReader', 'extract_source', 'extract_source_from_range', 'source_to_lines',
-    'join_source_list'
-]
+__all__ = ['Source', 'FortranReader', 'source_to_lines', 'join_source_list']
 
 
 class Source:
@@ -46,6 +43,18 @@ class Source:
         self.string = string
         self.file = file
 
+    def clone(self, **kwargs):
+        """
+        Replicate the object with the provided overrides.
+        """
+        if 'lines' not in kwargs:
+            kwargs['lines'] = self.lines
+        if self.string is not None and 'string' not in kwargs:
+            kwargs['string'] = self.string
+        if self.file is not None and 'file' not in kwargs:
+            kwargs['file'] = self.file
+        return type(self)(**kwargs)
+
     def __repr__(self):
         line_end = f'-{self.lines[1]}' if self.lines[1] else ''
         return f'Source<line {self.lines[0]}{line_end}>'
@@ -368,76 +377,6 @@ class FortranReader:
         return Source(lines=line.span, string='\n'.join(self.source_lines[start:end+1]))
 
 
-def extract_source(ast, text, label=None, full_lines=False):
-    """
-    Extract the marked string from source text.
-    """
-    attrib = getattr(ast, 'attrib', ast)
-    lstart = int(attrib['line_begin'])
-    lend = int(attrib['line_end'])
-    cstart = int(attrib['col_begin'])
-    cend = int(attrib['col_end'])
-    return extract_source_from_range((lstart, lend), (cstart, cend), text, label=label, full_lines=full_lines)
-
-
-def extract_source_from_range(lines, columns, text, label=None, full_lines=False):
-    """
-    Extract the marked string from source text.
-    """
-    text = text.splitlines(keepends=True)
-    lstart, lend = lines
-    cstart, cend = columns
-
-    if full_lines:
-        return Source(string=''.join(text[lstart-1:lend]).strip('\n'), lines=lines)
-
-    lines = text[lstart-1:lend]
-
-    # Scan for line continuations and honour inline
-    # comments in between continued lines
-    def continued(line):
-        if '!' in line:
-            line = line.split('!')[0]
-        return line.strip().endswith('&')
-
-    def is_comment(line):
-        return line.strip().startswith('!')
-
-    # We only honour line continuation if we're not parsing a comment
-    if not is_comment(lines[-1]):
-        while continued(lines[-1]) or is_comment(lines[-1]):
-            lend += 1
-            # TODO: Strip the leading empty space before the '&'
-            lines.append(text[lend-1])
-
-    # If line continuation is used, move column index to the relevant parts
-    while cstart >= len(lines[0]):
-        if not is_comment(lines[0]):
-            cstart -= len(lines[0])
-            cend -= len(lines[0])
-        lines = lines[1:]
-        lstart += 1
-
-    # Move column index by length of the label if given
-    if label is not None:
-        cstart += len(label)
-        cend += len(label)
-
-    # Avoid stripping indentation
-    if lines[0][:cstart].strip() == '':
-        cstart = 0
-
-    # TODO: The column indexes are still not right, so source strings
-    # for sub-expressions are likely wrong!
-    if lstart == lend:
-        lines[0] = lines[0][cstart:cend]
-    else:
-        lines[0] = lines[0][cstart:]
-        lines[-1] = lines[-1][:cend]
-
-    return Source(string=''.join(lines).strip('\n'), lines=(lstart, lend))
-
-
 def _merge_source_match_source(pre, match, post):
     """
     Merge a triple of :class:`Source`, :class:`re.Match`, :class:`Source` objects
