@@ -16,11 +16,13 @@ import pytest
 from loki import (
     Module, Subroutine, Sourcefile, BasicType, config, config_override
 )
-from loki.build import jit_compile, clean_test
+from loki.build import jit_compile
 from loki.expression import symbols as sym
-from loki.frontend import available_frontends, OMNI, FP
+from loki.frontend import available_frontends, OMNI, FP, HAVE_FP
 from loki.ir import nodes as ir, FindNodes, FindVariables
 
+from conftest import XFAIL_DERIVED_TYPE_JIT_TESTS
+
 
 @pytest.fixture(name='reset_frontend_mode')
 def fixture_reset_frontend_mode():
@@ -29,6 +31,10 @@ def fixture_reset_frontend_mode():
     config['frontend-strict-mode'] = original_frontend_mode
 
 
+@pytest.mark.xfail(
+    XFAIL_DERIVED_TYPE_JIT_TESTS,
+    reason='Support for user-defined derived type arguments is broken in JIT compile'
+)
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_check_alloc_opts(tmp_path, frontend):
     """
@@ -121,9 +127,11 @@ end module alloc_mod
     assert (item2.vector == 2.).all()
     mod.free_deferred(item2)
 
-    clean_test(filepath)
-
 
+@pytest.mark.xfail(
+    XFAIL_DERIVED_TYPE_JIT_TESTS,
+    reason='Support for user-defined derived type arguments is broken in JIT compile'
+)
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_associates(tmp_path, frontend):
     """Test the use of associate to access and modify other items"""
@@ -209,8 +217,6 @@ end module
     mod.associates(item)
     assert item.scalar == 17.0 and (item.vector == [1., 5., 10.]).all()
 
-    clean_test(filepath)
-
 
 @pytest.mark.parametrize('frontend', available_frontends(xfail=[(OMNI, 'OMNI fails to read without full module')]))
 def test_associates_deferred(frontend):
@@ -284,7 +290,6 @@ end subroutine associates_expr
     b = np.zeros(3, dtype='i')
     function(a, b)
     assert np.all(b == [7, 10, 13])
-    clean_test(filepath)
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -335,7 +340,6 @@ end subroutine test_enum
     function = jit_compile(routine, filepath=filepath, objname=routine.name)
     out = function()
     assert out == 23
-    clean_test(filepath)
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -782,7 +786,7 @@ module mod_variable_dimensions
   !$loki dimension(x+y,2*x)
   real(kind=jprb), allocatable :: v5(:,:)
   !$loki dimension(x/2, x**2, (x+y)/x)
-  real(kind=jprb), dimension(:, :, :), pointer :: v6 
+  real(kind=jprb), dimension(:, :, :), pointer :: v6
 end module mod_variable_dimensions
     """
 
@@ -1015,3 +1019,38 @@ end subroutine test_derived_type_parse
     assert isinstance(assigns[1].lhs, sym.Array)
     assert assigns[1].lhs.type.dtype == BasicType.REAL
     assert assigns[1].lhs.type.shape == (':',)
+
+
+@pytest.mark.skipif(not HAVE_FP, reason="Assumed size declarations only supported for FP")
+def test_assumed_size_declarations():
+    """
+    Test if assumed size declarations are correctly parsed.
+    """
+
+    fcode = """
+subroutine kernel(a, b, c)
+  implicit none
+  real, intent(in) :: a(*)
+  real, intent(in) :: b(8,*)
+  real, intent(in) :: c(8,0:*)
+
+end subroutine kernel
+"""
+
+    kernel = Subroutine.from_source(fcode, frontend=FP)
+
+    variable_map = kernel.variable_map
+    a = variable_map['a']
+    b = variable_map['b']
+    c = variable_map['c']
+
+    assert len(a.shape) == 1
+
+    assert len(b.shape) == 2
+    assert b.shape[0] == 8
+
+    assert len(c.shape) == 2
+    assert c.shape[0] == 8
+    assert c.shape[1].lower == 0
+
+    assert all('*' in str(shape) for shape in [a.shape, b.shape, c.shape])
