@@ -7,7 +7,7 @@
 
 from loki.expression import symbols as sym
 from loki.frontend import (
-    parse_omni_ast, parse_ofp_ast, parse_fparser_ast, get_fparser_node,
+    parse_omni_ast, parse_fparser_ast, get_fparser_node,
     parse_regex_source
 )
 from loki.ir import (
@@ -180,31 +180,6 @@ class Subroutine(ProgramUnit):
             type_map=type_map, scope=parent
         )
 
-    @classmethod
-    def from_ofp(cls, ast, raw_source, definitions=None, pp_info=None, parent=None):
-        """
-        Create :any:`Subroutine` from :any:`OFP` parse tree
-
-        Parameters
-        ----------
-        ast :
-            The OFP parse tree
-        raw_source : str
-            Fortran source string
-        definitions : list
-            List of external :any:`Module` to provide derived-type and procedure declarations
-        pp_info :
-            Preprocessing info as obtained by :any:`sanitize_input`
-        parent : :any:`Scope`, optional
-            The enclosing parent scope of the subroutine, typically a :any:`Module`.
-        """
-        if ast.tag not in ('subroutine', 'function'):
-            ast = [r for r in as_tuple(ast.find('file')) if r.tag in ('subroutine', 'function')].pop()
-        return parse_ofp_ast(
-            ast=ast, pp_info=pp_info, raw_source=raw_source,
-            definitions=definitions, scope=parent
-        )
-
     @classmethod
     def from_fparser(cls, ast, raw_source, definitions=None, pp_info=None, parent=None):
         """
