@@ -51,15 +51,10 @@ class FileWriteTransformation(Transformation):
         imports are honoured in the :any:`Scheduler` traversal.
         """
         if self.include_module_var_imports:
-            return (ProcedureItem, ModuleItem) #GlobalVariableItem)
+            return (ProcedureItem, ModuleItem)
         return ProcedureItem
 
-    def transform_file(self, sourcefile, **kwargs):
-        item = kwargs.get('item', None)
-        if not item and 'items' in kwargs:
-            if kwargs['items']:
-                item = kwargs['items'][0]
-
+    def _get_file_path(self, item, build_args):
         if not item:
             raise ValueError('No Item provided; required to determine file write path')
 
@@ -69,7 +64,26 @@ class FileWriteTransformation(Transformation):
         path = Path(item.path)
         suffix = self.suffix if self.suffix else path.suffix
         sourcepath = Path(item.path).with_suffix(f'.{_mode}{suffix}')
-        build_args = kwargs.get('build_args', {})
         if build_args and (output_dir := build_args.get('output_dir', None)) is not None:
             sourcepath = Path(output_dir)/sourcepath.name
+        return sourcepath
+
+    def transform_file(self, sourcefile, **kwargs):
+        item = kwargs.get('item')
+        if not item and 'items' in kwargs:
+            if kwargs['items']:
+                item = kwargs['items'][0]
+
+        build_args = kwargs.get('build_args', {})
+        sourcepath = self._get_file_path(item, build_args)
         sourcefile.write(path=sourcepath, cuf=self.cuf)
+
+    def plan_file(self, sourcefile, **kwargs):  # pylint: disable=unused-argument
+        item = kwargs.get('item')
+        if not item and 'items' in kwargs:
+            if kwargs['items']:
+                item = kwargs['items'][0]
+
+        build_args = kwargs.get('build_args', {})
+        sourcepath = self._get_file_path(item, build_args)
+        item.trafo_data['FileWriteTransformation'] = {'path': sourcepath}
