@@ -100,11 +100,13 @@ class GlobalVariableAnalysis(Transformation):
     def transform_subroutine(self, routine, **kwargs):
         if 'item' not in kwargs:
             raise RuntimeError('Cannot apply GlobalVariableAnalysis without item to store analysis data')
-        if 'successors' not in kwargs:
-            raise RuntimeError('Cannot apply GlobalVariableAnalysis without successors to store offload analysis data')
+        if 'sub_sgraph' not in kwargs:
+            raise RuntimeError(('Cannot apply GlobalVariableAnalysis without information'
+                ' about successors to store offload analysis data'))
 
         item = kwargs['item']
-        successors = kwargs['successors']
+        sub_sgraph = kwargs.get('sub_sgraph', None)
+        successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
 
         # Gather all symbols imported in this routine or parent scopes
         import_map = CaseInsensitiveDict()
@@ -289,7 +291,7 @@ class GlobalVarOffloadTransformation(Transformation):
         offload_variables = offload_variables - declared_variables
         if offload_variables:
             module.spec.append(
-                Pragma(keyword='acc', content=f'declare create({", ".join(v.name for v in offload_variables)})')
+                Pragma(keyword='loki', content=f'create device({", ".join(v.name for v in offload_variables)})')
             )
 
     def transform_subroutine(self, routine, **kwargs):
@@ -297,8 +299,9 @@ class GlobalVarOffloadTransformation(Transformation):
         Add data offload and pull-back directives to the driver
         """
         role = kwargs.get('role')
-        successors = kwargs.get('successors', ())
         item = kwargs['item']
+        sub_sgraph = kwargs.get('sub_sgraph', None)
+        successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
 
         if role == 'driver':
             self.process_driver(routine, successors)
@@ -380,11 +383,12 @@ class GlobalVarOffloadTransformation(Transformation):
             copyin_variables = {v for v, _ in uses_symbols if v.parent}
             if update_variables:
                 update_device += (
-                    Pragma(keyword='acc', content=f'update device({", ".join(v.name for v in update_variables)})'),
+                    Pragma(keyword='loki', content=f'update device({", ".join(v.name for v in update_variables)})'),
                 )
             if copyin_variables:
+                content = f'unstructured-data in({", ".join(v.name for v in copyin_variables)})'
                 update_device += (
-                    Pragma(keyword='acc', content=f'enter data copyin({", ".join(v.name for v in copyin_variables)})'),
+                    Pragma(keyword='loki', content=content),
                 )
 
         # All variables that are written in a kernel need a device-to-host transfer
@@ -397,15 +401,17 @@ class GlobalVarOffloadTransformation(Transformation):
             }
             if update_variables:
                 update_host += (
-                    Pragma(keyword='acc', content=f'update self({", ".join(v.name for v in update_variables)})'),
+                    Pragma(keyword='loki', content=f'update host({", ".join(v.name for v in update_variables)})'),
                 )
             if copyout_variables:
                 update_host += (
-                    Pragma(keyword='acc', content=f'exit data copyout({", ".join(v.name for v in copyout_variables)})'),
+                    Pragma(keyword='loki',
+                        content=f'end unstructured-data out({", ".join(v.name for v in copyout_variables)})'),
                 )
             if create_variables:
                 update_device += (
-                    Pragma(keyword='acc', content=f'enter data create({", ".join(v.name for v in create_variables)})'),
+                    Pragma(keyword='loki',
+                        content=f'unstructured-data create({", ".join(v.name for v in create_variables)})'),
                 )
 
         # Replace Loki pragmas with acc data/update pragmas
@@ -551,8 +557,9 @@ class GlobalVarHoistTransformation(Transformation):
         Hoist module variables.
         """
         role = kwargs.get('role')
-        successors = kwargs.get('successors', ())
         item = kwargs.get('item', None)
+        sub_sgraph = kwargs.get('sub_sgraph', None)
+        successors = sub_sgraph.successors(item) if sub_sgraph is not None else ()
 
         if role == 'driver':
             self.process_driver(routine, successors)
