@@ -9,11 +9,10 @@ import re
 from collections import  defaultdict
 
 from loki.batch import Transformation
-from loki.analyse import dataflow_analysis_attached
 from loki.expression import (
-    Quotient, IntLiteral, LogicLiteral, Variable, Array, Sum, Literal,
+    IntLiteral, LogicLiteral, Variable, Array, Sum, Literal,
     Product, InlineCall, Comparison, RangeIndex, Cast,
-    ProcedureSymbol, LogicalNot, simplify, is_dimension_constant,
+    ProcedureSymbol, simplify, is_dimension_constant,
     DetachScopesMapper
 )
 from loki.ir import (
@@ -51,9 +50,6 @@ class TemporariesPoolAllocatorTransformation(Transformation):
     * Inject Cray pointer assignments and stack pointer increments for all temporaries
     * Pass the local copy/copies of the stack integer(s) as argument to any nested kernel calls
 
-    By default, all local array arguments are allocated by the pool allocator, but this can be restricted
-    to include only those that have at least one dimension matching one of those provided in :data:`allocation_dims`.
-
     In a **driver** routine, the transformation will:
 
     * Determine the required scratch space from ``trafo_data``
@@ -71,15 +67,14 @@ class TemporariesPoolAllocatorTransformation(Transformation):
         SUBROUTINE DRIVER (...)
           ...
           INTEGER(KIND=8) :: ISTSZ
-          REAL, ALLOCATABLE :: ZSTACK(:, :)
+          REAL(KIND=REAL64), ALLOCATABLE :: ZSTACK(:, :)
           INTEGER(KIND=8) :: YLSTACK_L
           INTEGER(KIND=8) :: YLSTACK_U
-          ISTSZ = (C_SIZEOF(REAL(1, kind=jprb))*<array dim1>*<array dim2> + ...) / &
-           & C_SIZEOF(REAL(1, kind=JPRB))
+          ISTSZ = ISHFT(7 + C_SIZEOF(REAL(1, kind=jprb))**<array dim1>*<array dim2>, -3) + ...
           ALLOCATE (ZSTACK(ISTSZ, nb))
           DO b=1,nb
             YLSTACK_L = LOC(ZSTACK(1, b))
-            YLSTACK_U = YLSTACK_L + ISTSZ*C_SIZEOF(REAL(1, kind=JPRB))
+            YLSTACK_U = YLSTACK_L + ISTSZ*C_SIZEOF(REAL(1, kind=REAL64))
             CALL KERNEL(..., YDSTACK_L=YLSTACK_L, YDSTACK_U=YLSTACK_U)
           END DO
           DEALLOCATE (ZSTACK)
@@ -97,10 +92,10 @@ class TemporariesPoolAllocatorTransformation(Transformation):
           YLSTACK_L = YDSTACK_L
           YLSTACK_U = YDSTACK_U
           IP_tmp1 = YLSTACK_L
-          YLSTACK_L = YLSTACK_L + <array dim1>*<array dim2>*MAX(C_SIZEOF(REAL(1, kind=jprb)), 8)
+          YLSTACK_L = YLSTACK_L + ISHFT(ISHFT(<array dim1>*<array dim2>*C_SIZEOF(REAL(1, kind=JPRB)) + 7, -3), 3)
           IF (YLSTACK_L > YLSTACK_U) STOP
           IP_tmp2 = YLSTACK_L
-          YLSTACK_L = YLSTACK_L + ...*C_SIZEOF(REAL(1, kind=jprb))
+          YLSTACK_L = YLSTACK_L + ISHFT(ISHFT(...*C_SIZEOF(REAL(1, kind=JPRB)) + 7, -3), 3)
           IF (YLSTACK_L > YLSTACK_U) STOP
         END SUBROUTINE KERNEL
 
@@ -111,10 +106,10 @@ class TemporariesPoolAllocatorTransformation(Transformation):
         SUBROUTINE driver (NLON, NZ, NB, field1, field2)
           ...
           INTEGER(KIND=8) :: ISTSZ
-          REAL(KIND=JPRB), ALLOCATABLE :: ZSTACK(:, :)
+          REAL(KIND=REAL64), ALLOCATABLE :: ZSTACK(:, :)
           INTEGER(KIND=8) :: YLSTACK_L
           INTEGER(KIND=8) :: YLSTACK_U
-          ISTSZ = <array dim1>*<array dim2>
+          ISTSZ = ISTSZ = ISHFT(7 + C_SIZEOF(REAL(1, kind=jprb))**<array dim1>*<array dim2>, -3) + ...
           ALLOCATE (ZSTACK(ISTSZ, nb))
           DO b=1,nb
             YLSTACK_L = 1
@@ -130,17 +125,17 @@ class TemporariesPoolAllocatorTransformation(Transformation):
           INTEGER(KIND=8) :: YLSTACK_U
           INTEGER(KIND=8), INTENT(INOUT) :: YDSTACK_L
           INTEGER(KIND=8), INTENT(INOUT) :: YDSTACK_U
-          REAL(KIND=JPRB), CONTIGUOUS, INTENT(INOUT) :: ZSTACK(:)
+          REAL(KIND=REAL64), CONTIGUOUS, INTENT(INOUT) :: ZSTACK(:)
           POINTER(IP_tmp1, tmp1)
           POINTER(IP_tmp2, tmp2)
           ...
           YLSTACK_L = YDSTACK_L
           YLSTACK_U = YDSTACK_U
           IP_tmp1 = LOC(ZSTACK(YLSTACK_L))
-          YLSTACK_L = YLSTACK_L + <array dim1>*<array dim2>
+          YLSTACK_L = YLSTACK_L + ISHFT(<array dim1>*<array dim2>*C_SIZEOF(REAL(1, kind=JPRB)) + 7, -3)
           IF (YLSTACK_L > YLSTACK_U) STOP
           IP_tmp2 = LOC(ZSTACK(YLSTACK_L))
-          YLSTACK_L = YLSTACK_L + ...
+          YLSTACK_L = YLSTACK_L + ISHFT(...*C_SIZEOF(REAL(1, kind=JPRB)) + 7, -3)
           IF (YLSTACK_L > YLSTACK_U) STOP
         END SUBROUTINE KERNEL
 
@@ -222,18 +217,17 @@ class TemporariesPoolAllocatorTransformation(Transformation):
         ignore = item.ignore if item else ()
         targets = as_tuple(kwargs.get('targets', None))
 
-        self.stack_type_kind = 'JPRB'
         if item:
-            if (real_kind := item.config.get('real_kind', None)):
-                self.stack_type_kind = real_kind
-
             # Initialize set to store kind imports
             item.trafo_data[self._key] = {'kind_imports': {}}
 
         # add iso_c_binding import if necessary
         self.import_c_sizeof(routine)
+        # add iso_fortran_env import if necessary
+        self.import_real64(routine)
 
-        successors = kwargs.get('successors', ())
+        sub_sgraph = kwargs.get('sub_sgraph', None)
+        successors = as_tuple(sub_sgraph.successors(item)) if sub_sgraph is not None else ()
 
         if role == 'kernel':
             stack_size = self.apply_pool_allocator_to_temporaries(routine, item=item)
@@ -264,6 +258,20 @@ class TemporariesPoolAllocatorTransformation(Transformation):
             )
             routine.spec.prepend(imp)
 
+    @staticmethod
+    def import_real64(routine):
+        """
+        Import the real64 symbol if necesssary.
+        """
+
+        # add qualified iso_fortran_env import
+        if not 'REAL64' in routine.imported_symbols:
+            imp = Import(
+                module='ISO_FORTRAN_ENV', symbols=as_tuple(ProcedureSymbol('REAL64', scope=routine)),
+                nature='intrinsic'
+            )
+            routine.spec.prepend(imp)
+
     def import_allocation_types(self, routine, item):
         """
         Import all the variable types used in allocations.
@@ -394,29 +402,14 @@ class TemporariesPoolAllocatorTransformation(Transformation):
             stack_size_var = Variable(name=self.stack_size_name, type=stack_size_var_type)
 
             # Retrieve kind parameter of stack storage
-            _kind = routine.symbol_map.get(f'{self.stack_type_kind}', None) or Variable(name=self.stack_type_kind)
+            _kind = routine.symbol_map.get('REAL64', None) or Variable(name='REAL64')
 
             # Convert stack_size from bytes to integer
             stack_type_bytes = Cast(name='REAL', expression=Literal(1), kind=_kind)
             stack_type_bytes = InlineCall(Variable(name='C_SIZEOF'),
                                           parameters=as_tuple(stack_type_bytes))
-            if self.cray_ptr_loc_rhs:
-                stack_size_assign = Assignment(lhs=stack_size_var, rhs=stack_size)
-            else:
-                stack_size_assign = Assignment(lhs=stack_size_var, rhs=Quotient(stack_size, stack_type_bytes))
+            stack_size_assign = Assignment(lhs=stack_size_var, rhs=stack_size)
             body_prepend += [stack_size_assign]
-
-            # Stack-size no longer guaranteed to be a multiple of 8-bytes, so we have to check here
-            padding = Assignment(lhs=stack_size_var, rhs=Sum((stack_size_var, Literal(1))))
-            stack_size_check = Conditional(
-                condition=LogicalNot(Comparison(
-                    InlineCall(Variable(name='MOD'), parameters=(stack_size, stack_type_bytes)),
-                    '==', Literal(0))
-                ), inline=True, body=(padding,), else_body=None
-            )
-            if not self.cray_ptr_loc_rhs:
-                body_prepend += [stack_size_check]
-
             variables_append += [stack_size_var]
 
         if self.stack_storage_name in variable_map:
@@ -427,7 +420,7 @@ class TemporariesPoolAllocatorTransformation(Transformation):
             # allocation/deallocation statements
             stack_type = SymbolAttributes(
                 dtype=BasicType.REAL,
-                kind=Variable(name=self.stack_type_kind, scope=routine),
+                kind=Variable(name='REAL64', scope=routine),
                 shape=(RangeIndex((None, None)), RangeIndex((None, None))),
                 allocatable=True,
             )
@@ -443,14 +436,13 @@ class TemporariesPoolAllocatorTransformation(Transformation):
             stack_dealloc = Deallocation(variables=(stack_storage.clone(dimensions=None),))  # pylint: disable=no-member
 
             body_prepend += [stack_alloc]
-            if self.directive == 'openacc':
-                pragma_data_start = Pragma(
-                    keyword='acc',
-                    content=f'data create({stack_storage.name})' # pylint: disable=no-member
-                )
-                body_prepend += [pragma_data_start]
-                pragma_data_end = Pragma(keyword='acc', content='end data')
-                body_append += [pragma_data_end]
+            pragma_data_start = Pragma(
+                keyword='loki',
+                content=f'structured-data create({stack_storage.name})' # pylint: disable=no-member
+            )
+            body_prepend += [pragma_data_start]
+            pragma_data_end = Pragma(keyword='loki', content='end structured-data')
+            body_append += [pragma_data_end]
             body_append += [stack_dealloc]
 
         # Inject new variables and body nodes
@@ -615,29 +607,30 @@ class TemporariesPoolAllocatorTransformation(Transformation):
             ptr_assignment = Assignment(lhs=ptr_var, rhs=stack_ptr)
 
         # Build expression for array size in bytes
-        dim = arr.dimensions[0]
-        for d in arr.dimensions[1:]:
-            _dim = d
-            if isinstance(_dim, RangeIndex):
-                _dim = Sum((_dim.upper, Product((-1, _dim.lower)), 1))
-            dim = Product((dim, _dim))
+        dims = ()
+        for d in arr.dimensions:
+            if isinstance(d, RangeIndex):
+                dims += (Sum((d.upper, Product((-1, d.lower)), 1)),)
+            else:
+                dims += (d,)
+        dim = Product(dims)
         arr_type_bytes = InlineCall(Variable(name='C_SIZEOF'),
                                             parameters=as_tuple(self._get_c_sizeof_arg(arr)))
+        arr_size = Product((dim, arr_type_bytes))
 
-        # If the array size is not a multiple of NPROMA, then we pad the allocation to avoid
-        # potential alignment issues on device
-        if not self.horizontal or not any(s in dim for s in self.horizontal.sizes):
-            arr_type_bytes = InlineCall(function=Variable(name='MAX'),
-                        parameters=(arr_type_bytes, Literal(8)), kw_parameters=())
-        if self.cray_ptr_loc_rhs:
-            arr_size = dim
-        else:
-            arr_size = Product((dim, arr_type_bytes))
+        # Allocation is expressed in terms of REAL64, i.e., 8 byte values
+        # We obtain the allocation size by dividing the required size by 8 and rounding up,
+        # i.e., (size + 7) // 8, with the division implemented as bit shifts
+        ishift_func = InlineCall(function=Variable(name='ISHFT'))
+        arr_size = ishift_func.clone(parameters=(Sum((arr_size, 7)), -3))
 
         # Increment stack size
         stack_size = simplify(Sum((stack_size, arr_size)))
 
-        ptr_increment = Assignment(lhs=stack_ptr, rhs=Sum((stack_ptr, arr_size)))
+        if self.cray_ptr_loc_rhs:
+            ptr_increment = Assignment(lhs=stack_ptr, rhs=Sum((stack_ptr, arr_size)))
+        else:
+            ptr_increment = Assignment(lhs=stack_ptr, rhs=Sum((stack_ptr, ishift_func.clone(parameters=(arr_size, 3)))))
         if self.check_bounds:
             stack_size_check = Conditional(
                 condition=Comparison(stack_ptr, '>', stack_end), inline=True,
@@ -675,11 +668,15 @@ class TemporariesPoolAllocatorTransformation(Transformation):
         ]
 
         # Filter out unused vars
-        with dataflow_analysis_attached(routine):
-            temporary_arrays = [
+        #  this used to rely on dataflow_analysis only putting temporaries that are defined/written to
+        #  however, there exist some cases where temporaries are only read and it is not the
+        #  responsibility of this transformation to decide whether that is reasonable.
+        #  The following just removes unused temporaries so that they are not put on the stack.
+        used_vars = {v.name.lower() for v in FindVariables().visit(routine.body)}
+        temporary_arrays = [
                 var for var in temporary_arrays
-                if var.name.lower() in routine.body.defines_symbols
-            ]
+                if var.name.lower() in used_vars
+        ]
 
         # Filter out variables whose size is known at compile-time
         temporary_arrays = [
@@ -703,7 +700,7 @@ class TemporariesPoolAllocatorTransformation(Transformation):
         if self.cray_ptr_loc_rhs:
             stack_type = SymbolAttributes(
                     dtype=BasicType.REAL,
-                    kind=Variable(name=self.stack_type_kind, scope=routine),
+                    kind=Variable(name='REAL64', scope=routine),
                     shape=(RangeIndex((None, None)),), intent='inout', contiguous=True,
             )
             stack_storage = Variable(
@@ -760,35 +757,34 @@ class TemporariesPoolAllocatorTransformation(Transformation):
         stack_end = self._get_stack_end(routine)
 
         pragma_map = {}
-        if self.directive == 'openacc':
-            # Find OpenACC loop statements
-            acc_pragmas = [p for p in FindNodes(Pragma).visit(routine.body) if p.keyword.lower() == 'acc']
-            for pragma in acc_pragmas:
-                if pragma.content.lower().startswith('parallel') and 'gang' in pragma.content.lower():
-                    parameters = get_pragma_parameters(pragma, starts_with='parallel', only_loki_pragmas=False)
-                    if 'private' in [p.lower() for p in parameters]:
-                        var_end_str = f' {stack_var_end.name},' if self.check_bounds else ''
-                        content = re.sub(r'\bprivate\(', f'private({stack_var.name},{var_end_str} ',
-                                pragma.content.lower())
-                    else:
-                        var_end_str = f', {stack_var_end.name}' if self.check_bounds else ''
-                        content = pragma.content + f' private({stack_var.name}{var_end_str})'
-                    pragma_map[pragma] = pragma.clone(content=content)
-
-        elif self.directive == 'openmp':
-            # Find OpenMP parallel statements
-            omp_pragmas = [p for p in FindNodes(Pragma).visit(routine.body) if p.keyword.lower() == 'omp']
-            for pragma in omp_pragmas:
-                if pragma.content.lower().startswith('parallel'):
-                    parameters = get_pragma_parameters(pragma, starts_with='parallel', only_loki_pragmas=False)
-                    if 'private' in [p.lower() for p in parameters]:
-                        var_end_str = f' {stack_var_end.name},' if self.check_bounds else ''
-                        content = re.sub(r'\bprivate\(', f'private({stack_var.name},{var_end_str}',
-                                pragma.content.lower())
-                    else:
-                        var_end_str = f', {stack_var_end.name}' if self.check_bounds else ''
-                        content = pragma.content + f' private({stack_var.name}{var_end_str})'
-                    pragma_map[pragma] = pragma.clone(content=content)
+        pragmas = [p for p in FindNodes(Pragma).visit(routine.body) if p.keyword.lower() == 'loki']
+        for pragma in pragmas:
+            if pragma.content.lower().startswith('loop gang'):
+                parameters = get_pragma_parameters(pragma, starts_with='loop gang', only_loki_pragmas=False)
+                if 'private' in [p.lower() for p in parameters]:
+                    var_end_str = f' {stack_var_end.name},' if self.check_bounds else ''
+                    content = re.sub(r'\bprivate\(', f'private({stack_var.name},{var_end_str} ',
+                            pragma.content.lower())
+                else:
+                    var_end_str = f', {stack_var_end.name}' if self.check_bounds else ''
+                    content = pragma.content + f' private({stack_var.name}{var_end_str})'
+                pragma_map[pragma] = pragma.clone(content=content)
+        # problem being that code, like e.g. ecwam transformed for 'idem-stack', already having
+        #  OpenMP pragmas rely on the following. Once we (decide to) implement a
+        #  'reverse PragmaModel' trafo that converts e.g., OpenMP pragmas to generic Loki pragmas
+        #  we do not longer rely on the following
+        omp_pragmas = [p for p in FindNodes(Pragma).visit(routine.body) if p.keyword.lower() == 'omp']
+        for pragma in omp_pragmas:
+            if pragma.content.lower().startswith('parallel'):
+                parameters = get_pragma_parameters(pragma, starts_with='parallel', only_loki_pragmas=False)
+                if 'private' in [p.lower() for p in parameters]:
+                    var_end_str = f' {stack_var_end.name},' if self.check_bounds else ''
+                    content = re.sub(r'\bprivate\(', f'private({stack_var.name},{var_end_str}',
+                            pragma.content.lower())
+                else:
+                    var_end_str = f', {stack_var_end.name}' if self.check_bounds else ''
+                    content = pragma.content + f' private({stack_var.name}{var_end_str})'
+                pragma_map[pragma] = pragma.clone(content=content)
 
         if pragma_map:
             routine.body = Transformer(pragma_map).visit(routine.body)
@@ -841,9 +837,7 @@ class TemporariesPoolAllocatorTransformation(Transformation):
                 )
 
             # Retrieve kind parameter of stack storage
-            _kind = (routine.imported_symbol_map.get(f'{self.stack_type_kind}', None) or
-                     routine.variable_map.get(f'{self.stack_type_kind}', None) or
-                     Variable(name=self.stack_type_kind))
+            _kind = routine.imported_symbol_map.get('REAL64')
 
             # Stack increment
             if self.cray_ptr_loc_rhs:
