@@ -66,8 +66,8 @@ class SCCHoistTemporaryArraysTransformation(HoistVariablesTransformation):
         # Add explicit device-side allocations/deallocations for hoisted temporaries
         vnames = ', '.join(v.name for v in variables)
         if vnames:
-            pragma = ir.Pragma(keyword='acc', content=f'enter data create({vnames})')
-            pragma_post = ir.Pragma(keyword='acc', content=f'exit data delete({vnames})')
+            pragma = ir.Pragma(keyword='loki', content=f'unstructured-data create({vnames})')
+            pragma_post = ir.Pragma(keyword='loki', content=f'end unstructured-data delete({vnames})')
 
             # Add comments around standalone pragmas to avoid false attachment
             routine.body.prepend((ir.Comment(''), pragma, ir.Comment('')))
