@@ -7,9 +7,9 @@
 
 import pytest
 
-from loki import Sourcefile, Dimension, fgen
+from loki import Sourcefile, Dimension, fgen, SGraph
 from loki.batch import (
-    Scheduler, SchedulerConfig, ProcedureItem, ModuleItem
+    Scheduler, SchedulerConfig, ProcedureItem
 )
 from loki.frontend import available_frontends
 from loki.ir import (
@@ -18,12 +18,13 @@ from loki.ir import (
 )
 
 from loki.transformations import (
-    InlineTransformation
+    InlineTransformation, PragmaModelTransformation
 )
 from loki.transformations.single_column import (
     SCCBaseTransformation, SCCDevectorTransformation,
     SCCDemoteTransformation, SCCRevectorTransformation,
     SCCAnnotateTransformation, SCCHoistPipeline,
+    SCCVHoistPipeline, SCCSHoistPipeline
 )
 
 
@@ -45,7 +46,8 @@ def fixture_blocking():
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking):
+@pytest.mark.parametrize('hoist_pipeline', [SCCVHoistPipeline, SCCSHoistPipeline])
+def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking, hoist_pipeline):
     """
     Test hoisting of column temporaries to "driver" level.
     """
@@ -99,24 +101,32 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking):
     driver_item = ProcedureItem(name='#column_driver', source=driver_source)
     kernel_item = ProcedureItem(name='#compute_column', source=kernel_source)
 
-    scc_hoist = SCCHoistPipeline(
+    scc_hoist = hoist_pipeline(
         horizontal=horizontal, block_dim=blocking, directive='openacc'
     )
 
+    graph_dic = {driver_item: [kernel_item]}
+    graph = SGraph.from_dict(graph_dic)
     # Apply pipeline in reverse order to ensure analysis runs before hoisting
     scc_hoist.apply(kernel, role='kernel', item=kernel_item)
     scc_hoist.apply(
         driver, role='driver', item=driver_item,
-        successors=(kernel_item,), targets=['compute_column']
+        sub_sgraph=graph,
+        targets=['compute_column']
     )
 
     # Ensure we two loops left in kernel
     kernel_loops = FindNodes(Loop).visit(kernel.body)
-    assert len(kernel_loops) == 2
-    assert kernel_loops[0].variable == 'jl'
-    assert kernel_loops[0].bounds == 'start:end'
-    assert kernel_loops[1].variable == 'jk'
-    assert kernel_loops[1].bounds == '2:nz'
+    if hoist_pipeline == SCCSHoistPipeline:
+        assert len(kernel_loops) == 1
+        assert kernel_loops[0].variable == 'jk'
+        assert kernel_loops[0].bounds == '2:nz'
+    else:
+        assert len(kernel_loops) == 2
+        assert kernel_loops[0].variable == 'jl'
+        assert kernel_loops[0].bounds == 'start:end'
+        assert kernel_loops[1].variable == 'jk'
+        assert kernel_loops[1].bounds == '2:nz'
 
     # Ensure all expressions and array indices are unchanged
     assigns = FindNodes(Assignment).visit(kernel.body)
@@ -126,7 +136,20 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking):
 
     # Ensure we have only one driver block loop
     driver_loops = FindNodes(Loop).visit(driver.body)
-    assert len(driver_loops) == 1
+    if hoist_pipeline == SCCSHoistPipeline:
+        assert len(driver_loops) == 3
+        assert driver_loops[1].variable == 'jl'
+        assert driver_loops[1].bounds == 'start:end'
+        calls = FindNodes(CallStatement).visit(driver_loops[1])
+        assert len(calls) == 1
+        assert calls[0].name == 'compute_column'
+        assert driver_loops[2].variable == 'jl'
+        assert driver_loops[2].bounds == 'start:end'
+        calls = FindNodes(CallStatement).visit(driver_loops[2])
+        assert len(calls) == 1
+        assert calls[0].name == 'compute_column'
+    else:
+        assert len(driver_loops) == 1
     assert driver_loops[0].variable == 'b'
     assert driver_loops[0].bounds == '1:nb'
 
@@ -257,11 +280,12 @@ END MODULE kernel_mod
     driver = scheduler["#driver"].ir
     kernel = scheduler["kernel_mod#kernel"].ir
 
-    transformation = (SCCBaseTransformation(horizontal=horizontal, directive='openacc'),)
+    transformation = (SCCBaseTransformation(horizontal=horizontal),)
     transformation += (SCCDevectorTransformation(horizontal=horizontal, trim_vector_sections=trim_vector_sections),)
     transformation += (SCCDemoteTransformation(horizontal=horizontal),)
     transformation += (SCCRevectorTransformation(horizontal=horizontal),)
-    transformation += (SCCAnnotateTransformation(directive='openacc', block_dim=blocking),)
+    transformation += (SCCAnnotateTransformation(block_dim=blocking),)
+    transformation += (PragmaModelTransformation(directive='openacc'),)
     for transform in transformation:
         scheduler.process(transformation=transform)
 
@@ -429,17 +453,19 @@ end module my_scaling_value_mod
 
     driver_item = ProcedureItem(name='#column_driver', source=driver_source)
     kernel_item = ProcedureItem(name='#compute_column', source=kernel_source)
-    module_item = ModuleItem(name='my_scaling_value_mod', source=module_source)
 
     scc_hoist = SCCHoistPipeline(
         horizontal=horizontal, block_dim=blocking,
         directive='openacc', dim_vars=vertical.sizes
     )
 
+    graph_dic = {driver_item: [kernel_item]}
+    graph = SGraph.from_dict(graph_dic)
     # Apply in reverse order to ensure hoisting analysis gets run on kernel first
-    scc_hoist.apply(kernel, role='kernel', item=kernel_item, successors=(module_item,))
+    scc_hoist.apply(kernel, role='kernel', item=kernel_item)
     scc_hoist.apply(
-        driver, role='driver', item=driver_item, successors=(kernel_item,), targets=['compute_column']
+        driver, role='driver', item=driver_item, targets=['compute_column'],
+        sub_sgraph=graph
     )
 
     # Check that blocking size has not been redefined
@@ -560,15 +586,17 @@ def test_scc_hoist_nested_openacc(frontend, horizontal, vertical, blocking,
         dim_vars=vertical.sizes, as_kwarguments=as_kwarguments, directive='openacc'
     )
 
+    graph_dic = {driver_item: [outer_kernel_item], outer_kernel_item: [inner_kernel_item]}
+    graph = SGraph.from_dict(graph_dic)
     # Apply in reverse order to ensure hoisting analysis gets run on kernel first
     scc_hoist.apply(inner_kernel, role='kernel', item=inner_kernel_item)
     scc_hoist.apply(
         outer_kernel, role='kernel', item=outer_kernel_item,
-        targets=['compute_q'], successors=(inner_kernel_item,)
+        targets=['compute_q'], sub_sgraph=graph.get_sub_sgraph(outer_kernel_item)
     )
     scc_hoist.apply(
         driver, role='driver', item=driver_item,
-        targets=['compute_column'], successors=(outer_kernel_item,)
+        targets=['compute_column'], sub_sgraph=graph
     )
 
     # Ensure calls have correct arguments
@@ -741,15 +769,18 @@ def test_scc_hoist_nested_inline_openacc(frontend, horizontal, vertical, blockin
 
     InlineTransformation(allowed_aliases=horizontal.index).apply(outer_kernel)
 
+    graph_dic = {driver_item: [outer_kernel_item], outer_kernel_item: [inner_kernel_item]}
+    graph = SGraph.from_dict(graph_dic)
+
     # Apply in reverse order to ensure hoisting analysis gets run on kernel first
     scc_hoist.apply(inner_kernel, role='kernel', item=inner_kernel_item)
     scc_hoist.apply(
         outer_kernel, role='kernel', item=outer_kernel_item,
-        targets=['compute_q'], successors=()
+        targets=['compute_q']
     )
     scc_hoist.apply(
         driver, role='driver', item=driver_item,
-        targets=['compute_column'], successors=(outer_kernel_item,)
+        targets=['compute_column'], sub_sgraph=graph
     )
 
     # Ensure calls have correct arguments
